@@ -29,6 +29,7 @@ stateProbs <- function(m)
 
     data <- m$data
     nbStates <- ncol(m$mle$stepPar)
+    nbAnimals <- length(unique(data$ID))
 
     if(nbStates==1)
         stop("No states to decode (nbStates=1)")
@@ -36,12 +37,19 @@ stateProbs <- function(m)
     nbObs <- nrow(data)
     la <- logAlpha(m) # forward log-probabilities
     lb <- logBeta(m) # backward log-probabilities
-    c <- max(la[nbObs,]) # cancels out below ; prevents numerical errors
-    llk <- c + log(sum(exp(la[nbObs,]-c)))
     stateProbs <- matrix(NA,nbObs,nbStates)
 
-    for(i in 1:nbObs)
+    aInd <- NULL
+    for(i in 1:nbAnimals)
+        aInd <- c(aInd,max(which(data$ID==unique(data$ID)[i])))
+
+    for(i in nbObs:1){
+        if(any(i==aInd)){
+            c <- max(la[i,]) # cancels out below ; prevents numerical errors
+            llk <- c + log(sum(exp(la[i,]-c)))
+        }
         stateProbs[i,] <- exp(la[i,]+lb[i,]-llk)
+    }
 
     return(stateProbs)
 }
